@@ -1,7 +1,7 @@
 // qmidictlAbout.h
 //
 /****************************************************************************
-   Copyright (C) 2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2010-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,17 +22,20 @@
 #ifndef __qmidictlAbout_h
 #define __qmidictlAbout_h
 
+#if !defined(__WIN32__) && !defined(_WIN32) && !defined(WIN32) && !defined(Q_OS_SYMBIAN)
 #include "config.h"
-
 #define QMIDICTL_TITLE        PACKAGE_NAME
-#define QMIDICTL_VERSION      PACKAGE_VERSION
+#else
+#define QMIDICTL_TITLE        "QmidiCtl"
+#endif
 
 #define QMIDICTL_SUBTITLE     "A MIDI Remote Controller via UDP/IP Multicast"
-#define QMIDICTL_WEBSITE      "http://qmidictl.sourceforge.net"
-#define QMIDICTL_COPYRIGHT    "Copyright (C) 2010, rncbc aka Rui Nuno Capela. All rights reserved."
+#define QMIDICTL_WEBSITE      "https://qmidictl.sourceforge.io"
+#define QMIDICTL_COPYRIGHT    "Copyright (C) 2010-2023, rncbc aka Rui Nuno Capela. All rights reserved."
 
 #define QMIDICTL_DOMAIN       "rncbc.org"
 
 #endif  // __qmidictlAbout_h
 
 // end of qmidictlAbout.h
+
