@@ -1,7 +1,7 @@
 // qmidictlMainForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2010-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,10 +30,15 @@
 #include "qmidictlUdpDevice.h"
 #include "qmidictlDialStyle.h"
 
-#include <QTimer>
+#if defined(Q_OS_ANDROID)
+#include "qmidictlActionBar.h"
+#endif
 
+#include <QTimer>
 #include <QMessageBox>
 
+#include <QTouchEvent>
+
 
 // Possible cubic-root optimization.
 // (borrowed from metamerist.com)
@@ -62,22 +67,23 @@ static inline float cubef2 ( float x )
 // qmidictlMainForm -- UI wrapper form.
 //
 
-qmidictlMainForm *qmidictlMainForm::g_pMainForm = NULL;
+qmidictlMainForm *qmidictlMainForm::g_pMainForm = nullptr;
 
 // Constructor.
-qmidictlMainForm::qmidictlMainForm (
-	QWidget *pParent, Qt::WindowFlags wflags )
-	: QMainWindow(pParent, wflags)
+qmidictlMainForm::qmidictlMainForm ( QWidget *pParent )
+	: QMainWindow(pParent)
 {
 	// Setup UI struct...
 	m_ui.setupUi(this);
-
+#if QT_VERSION < QT_VERSION_CHECK(6, 1, 0)
+	QMainWindow::setWindowIcon(QIcon(":/images/qmidictl.png"));
+#endif
 	// The main network device.
 	m_pUdpDevice = new qmidictlUdpDevice(this);
 
 	// Strip page/states.
 	m_iStripPages = 4;
-	m_pStripStates = NULL;
+	m_pStripStates = nullptr;
 	m_iCurrentStripPage = 0;
 
 	// Jog-wheel last known state.
@@ -95,6 +101,51 @@ qmidictlMainForm::qmidictlMainForm (
 	m_pDialStyle = new qmidictlDialStyle();
 	m_ui.jogWheelDial->setStyle(m_pDialStyle);
 
+	// Enable multi-gestures...
+	m_ui.MainCentralWidget->setAttribute(Qt::WA_AcceptTouchEvents);
+	m_ui.MainCentralWidget->installEventFilter(this);
+
+#if defined(Q_OS_ANDROID)
+
+	// Special stuff for the android platform.
+	m_ui.optionsAction->setIcon(QIcon(":/images/actionOptions.png"));
+	m_ui.configureAction->setIcon(QIcon(":/images/actionConfigure.png"));
+	m_ui.aboutAction->setIcon(QIcon(":/images/actionAbout.png"));
+//	m_ui.exitAction->setIcon(QIcon(":/images/actionCancel.png"));
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 1, 0)
+	m_ui.prevStripPageButton->setMaximumSize(96, 48);
+	m_ui.nextStripPageButton->setMaximumSize(96, 48);
+#endif
+
+	// Special action-bar for the android stuff.
+	m_pActionBar = new qmidictlActionBar();
+	m_pActionBar->setIcon(QIcon(":/images/qmidictl.png"));
+#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
+	m_pActionBar->setTitle(QApplication::applicationDisplayName());
+#else
+	m_pActionBar->setTitle(QMIDICTL_TITLE " - " + tr(QMIDICTL_SUBTITLE));
+#endif
+#if QT_VERSION < QT_VERSION_CHECK(6, 1, 0)
+	// Action-bar left-drop-down menu items...
+	m_pActionBar->addMenuItem(m_ui.optionsAction);
+	m_pActionBar->addMenuItem(m_ui.configureAction);
+	m_pActionBar->addMenuItem(m_ui.aboutAction);
+//	m_pActionBar->addMenuItem(m_ui.exitAction);
+#endif
+	// Action-bar right-overflow button items...
+	m_pActionBar->addButton(m_ui.optionsAction);
+	m_pActionBar->addButton(m_ui.configureAction);
+	m_pActionBar->addButton(m_ui.aboutAction);
+//	m_pActionBar->addButton(m_ui.exitAction);
+	// Make it at the top...
+	m_ui.MainCentralLayout->insertWidget(0, m_pActionBar);
+
+#endif
+
+	// Custom swipe gesture trackers.
+	m_bSwipe = false;
+
 	// Pseudo-singleton reference setup.
 	g_pMainForm = this;
 
@@ -148,14 +199,19 @@ qmidictlMainForm::qmidictlMainForm (
 
 	// Network signal/slot connections...
 	QObject::connect(m_pUdpDevice,
-		SIGNAL(received(const QByteArray&)),
-		SLOT(receiveSlot(const QByteArray&)));
+		SIGNAL(received(QByteArray)),
+		SLOT(receiveSlot(QByteArray)));
 }
 
 
 // Destructor.
 qmidictlMainForm::~qmidictlMainForm (void)
 {
+#if defined(Q_OS_ANDROID)
+	// No need for special android stuff anymore.
+	delete m_pActionBar;
+#endif
+
 	// No need for fancy styling no more.
 	delete m_pDialStyle;
 
@@ -166,7 +222,7 @@ qmidictlMainForm::~qmidictlMainForm (void)
 	delete m_pUdpDevice;
 
 	// Pseudo-singleton reference shut-down.
-	g_pMainForm = NULL;
+	g_pMainForm = nullptr;
 }
 
 
@@ -202,12 +258,15 @@ int qmidictlMainForm::currentStripPage (void) const
 void qmidictlMainForm::setup (void)
 {
 	qmidictlOptions *pOptions = qmidictlOptions::getInstance();
-	if (pOptions == NULL)
+	if (pOptions == nullptr)
 		return;
 
-	if (!m_pUdpDevice->open(pOptions->sInterface, pOptions->iUdpPort)) {
+	if (!m_pUdpDevice->open(
+			pOptions->sInterface,
+			pOptions->sUdpAddr,
+			pOptions->iUdpPort)) {
 		QMessageBox::critical(this,
-			tr("Network Inferface Error - %1").arg(QMIDICTL_TITLE),
+			tr("Network Inferface Error"),
 			tr("The network interface could not be established.\n\n"
 			"Please, make sure you have an on-line network connection "
 			"and try again."));
@@ -359,7 +418,7 @@ void qmidictlMainForm::sendMmcCommand (
 	sendData(pSysex, iSysex);
 
 	// Done.
-	delete pSysex;
+	delete [] pSysex;
 }
 
 
@@ -469,7 +528,7 @@ void qmidictlMainForm::sendData ( unsigned char *data, unsigned short len )
 void qmidictlMainForm::sendCommand ( int iCommand, int iTrack, int iValue )
 {
 	qmidictlMidiControl *pMidiControl = qmidictlMidiControl::getInstance();
-	if (pMidiControl == NULL)
+	if (pMidiControl == nullptr)
 		return;
 	
 	qmidictlMidiControl::Command command
@@ -687,7 +746,7 @@ void qmidictlMainForm::forwardSlot ( bool bOn )
 
 
 // Network listener/receiver slot.
-void qmidictlMainForm::receiveSlot ( const QByteArray& data )
+void qmidictlMainForm::receiveSlot ( QByteArray data )
 {
 	recvData((unsigned char *) data.constData(), data.length());
 }
@@ -695,7 +754,7 @@ void qmidictlMainForm::receiveSlot ( const QByteArray& data )
 void qmidictlMainForm::recvData ( unsigned char *data, unsigned short len )
 {
 	qmidictlMidiControl *pMidiControl = qmidictlMidiControl::getInstance();
-	if (pMidiControl == NULL)
+	if (pMidiControl == nullptr)
 		return;
 
 	m_iBusy++;
@@ -847,10 +906,8 @@ void qmidictlMainForm::recvData ( unsigned char *data, unsigned short len )
 		if (key.isLogarithmic())
 			iValue = int(127.0f * cubef2(float(iValue) / 127.0f));
 	#ifdef CONFIG_DEBUG
-		qDebug("recvData: Command=\"%s\" Type=\"%s\" Track=%d Value=%d",
-			qmidictlMidiControl::textFromCommand(command).toUtf8().constData(),
-			qmidictlMidiControl::textFromType(ctype).toUtf8().constData(),
-			iTrack, iValue);
+		qDebug("recvData: Command=0x%04x Type=0x%04x Track=%d Value=%d",
+			uint(command), uint(ctype), iTrack, iValue);
 	#endif
 		switch (command) {
 		case qmidictlMidiControl::RST:
@@ -950,23 +1007,34 @@ void qmidictlMainForm::configureSlot (void)
 void qmidictlMainForm::aboutSlot (void)
 {
 	// Stuff the about box text...
-	QString sText = "<p>\n";
-	sText += "<b>" QMIDICTL_TITLE " - " + tr(QMIDICTL_SUBTITLE) + "</b><br />\n";
+	QString sText = "<h1>" QMIDICTL_TITLE "</h1>\n";
+	sText += "<p>" + tr(QMIDICTL_SUBTITLE) + "<br />\n";
+	sText += "<br />\n";
+	sText += tr("Version") + ": <b>" CONFIG_BUILD_VERSION "</b><br />\n";
+//	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
+#ifdef CONFIG_DEBUG
+	sText += "<small><font color=\"red\">";
+	sText += tr("Debugging option enabled.");
+	sText += "</font></small>\n";
+#endif
+	sText += "<br />\n";
+	sText += tr("Using: Qt %1").arg(qVersion());
+#if defined(QT_STATIC)
+	sText += "-static";
+#endif
 	sText += "<br />\n";
-	sText += tr("Version") + ": <b>" QMIDICTL_VERSION "</b><br />\n";
-	sText += "<small>" + tr("Build") + ": " __DATE__ " " __TIME__ "</small><br />\n";
 	sText += "<br />\n";
 	sText += tr("Website") + ": <a href=\"" QMIDICTL_WEBSITE "\">" QMIDICTL_WEBSITE "</a><br />\n";
 	sText += "<br />\n";
 	sText += "<small>";
 	sText += QMIDICTL_COPYRIGHT "<br />\n";
 	sText += "<br />\n";
-	sText += tr("This program is free software; you can redistribute it and/or modify it") + "<br />\n";
+	sText += tr("This program is free software; you can redistribute it and/or modify it") + '\n';
 	sText += tr("under the terms of the GNU General Public License version 2 or later.");
 	sText += "</small>";
 	sText += "</p>\n";
 
-	QMessageBox::about(this, tr("About %1").arg(QMIDICTL_TITLE), sText);
+	QMessageBox::about(this, tr("About"), sText);
 }
 
 
@@ -977,4 +1045,239 @@ void qmidictlMainForm::exitSlot (void)
 }
 
 
+// Provided for multi-touch support...
+bool qmidictlMainForm::eventFilter ( QObject *pObject, QEvent *pEvent )
+{
+	if (pObject == m_ui.MainCentralWidget) {
+		if (pEvent->type() == QEvent::TouchBegin  ||
+			pEvent->type() == QEvent::TouchUpdate ||
+			pEvent->type() == QEvent::TouchEnd    ||
+			pEvent->type() == QEvent::TouchCancel) {
+			QTouchEvent *pTouchEvent = static_cast<QTouchEvent *> (pEvent);
+			if (pTouchEvent)
+				return touchEvent(pTouchEvent);
+		}
+	}
+
+	return QMainWindow::eventFilter(pObject, pEvent);
+}
+
+
+bool qmidictlMainForm::touchEvent ( QTouchEvent *pTouchEvent )
+{
+	// Make up with multi-touch stuff...
+	// -- synthesize mouse events:
+	int iTouched = 0;
+
+	const QWidget *pCentralWidget
+		= m_ui.MainCentralWidget;
+
+	QEvent::Type etype;
+	QMouseEvent *pMouseEvent;
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+	const QList<QEventPoint>& points
+		= pTouchEvent->points();
+#else
+	const QList<QTouchEvent::TouchPoint>& points
+		= pTouchEvent->touchPoints();
+#endif
+
+	switch (pTouchEvent->type()) {
+	case QEvent::TouchBegin:
+		// Begin...
+		m_touched.clear();
+		if (points.size() == 1) {
+		#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+			const QEventPoint& point = points.at(0);
+		#else
+			const QTouchEvent::TouchPoint& point = points.at(0);
+		#endif
+			// Replay as first mouse-press...
+			const int id = point.id();
+		#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+			const QPoint& pos = point.position().toPoint();
+		#else
+			const QPoint& pos = point.pos().toPoint();
+		#endif
+			const QList<QWidget *>& children
+				= pCentralWidget->findChildren<QWidget *> ();
+			foreach (QWidget *pWidget, children) {
+				const QPoint& wpos = pWidget->mapFrom(pCentralWidget, pos);
+				if (pWidget->rect().contains(wpos) &&
+					pWidget->childAt(wpos) == nullptr) {
+					m_touched.insert(id, pWidget);
+					etype = QEvent::MouseButtonPress;
+					pMouseEvent = new QMouseEvent(
+						etype, wpos, pos,
+						Qt::LeftButton,
+						Qt::LeftButton,
+						Qt::NoModifier);
+					QApplication::postEvent(pWidget, pMouseEvent);
+					++iTouched;
+				}
+			}
+			// Start tracking a swipe gesture...
+			m_bSwipe = true;
+		}
+		// Fall thru...
+	case QEvent::TouchUpdate:
+		// Whether currently tracking a swipe gesture...
+		if (m_bSwipe) {
+		#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+			const QEventPoint& point = points.at(0);
+			const QPointF& p0 = point.globalPressPosition();
+			const QPointF& p1 = point.globalLastPosition();
+			const QPointF& p2 = point.globalPosition();
+		#else
+			const QTouchEvent::TouchPoint& point = points.at(0);
+			const QPointF& p0 = point.startScreenPos();
+			const QPointF& p1 = point.lastScreenPos();
+			const QPointF& p2 = point.screenPos();
+		#endif
+			const qreal dx2 = (p2.x() - p1.x()) * (p1.x() - p0.x());
+			const qreal dy = p2.y() - p0.y();
+			if (dx2 < 0.0 || qAbs(dy) > 60.0 || points.size() > 1) {
+				// And cancel swipe...
+				m_bSwipe = false;
+			}
+			// Done with swipe tracking, for now...
+			break;
+		}
+		// Assess all touch-point-ids...
+	#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+		foreach (QEventPoint point, points) {
+			const QPoint& pos = point.position().toPoint();
+	#else
+		foreach (QTouchEvent::TouchPoint point, points) {
+			const QPoint& pos = point.pos().toPoint();
+	#endif
+			const int id = point.id();
+			QWidget *pTouchedWidget = m_touched.value(id, nullptr);
+			if (pTouchedWidget) {
+				const QPoint& wpos
+					= pTouchedWidget->mapFrom(pCentralWidget, pos);
+			#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+				if (point.state() & QEventPoint::Released) {
+			#else
+				if (point.state() & Qt::TouchPointReleased) {
+			#endif
+					m_touched.remove(id);
+					etype = QEvent::MouseButtonRelease;
+				} else {
+					etype = QEvent::MouseMove;
+				}
+				pMouseEvent = new QMouseEvent(
+					etype, wpos, pos,
+					Qt::LeftButton,
+					Qt::LeftButton,
+					Qt::NoModifier);
+				QApplication::postEvent(pTouchedWidget, pMouseEvent);
+				++iTouched;
+			} else {
+				const QList<QWidget *>& children
+					= pCentralWidget->findChildren<QWidget *> ();
+				foreach (QWidget *pWidget, children) {
+					const QPoint& wpos = pWidget->mapFrom(pCentralWidget, pos);
+					if (pWidget->rect().contains(wpos) &&
+						pWidget->childAt(wpos) == nullptr) {
+						m_touched.insert(id, pWidget);
+						etype = QEvent::MouseButtonPress;
+						pMouseEvent = new QMouseEvent(
+							etype, wpos, pos,
+							Qt::LeftButton,
+							Qt::LeftButton,
+							Qt::NoModifier);
+						QApplication::postEvent(pWidget, pMouseEvent);
+						++iTouched;
+						break;
+					}
+				}
+			}
+		}
+		break;
+	case QEvent::TouchEnd:
+		// Whether currently tracking some swipe gesture...
+		if (m_bSwipe) {
+		#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+			const QEventPoint& point = points.at(0);
+			const QPointF& p0 = point.globalPressPosition();
+			const QPointF& p2 = point.globalPosition();
+		#else
+			const QTouchEvent::TouchPoint& point = points.at(0);
+			const QPointF& p0 = point.startScreenPos();
+			const QPointF& p2 = point.screenPos();
+		#endif
+			const qreal dx = p2.x() - p0.x();
+			if (qAbs(dx) > 60.0) {
+				// Perform swipe...
+				if (dx < 0.0)
+					nextStripPageSlot();
+				else
+					prevStripPageSlot();
+				// Done swipe.
+			} else {
+				// Replay...
+			#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+				const QPoint& pos = point.position().toPoint();
+			#else
+				const QPoint& pos = point.pos().toPoint();
+			#endif
+				const int id = point.id();
+				const QList<QWidget *>& children
+					= pCentralWidget->findChildren<QWidget *> ();
+				foreach (QWidget *pWidget, children) {
+					const QPoint& wpos = pWidget->mapFrom(pCentralWidget, pos);
+					if (pWidget->rect().contains(wpos) &&
+						pWidget->childAt(wpos) == nullptr) {
+						m_touched.insert(id, pWidget);
+						etype = QEvent::MouseButtonPress;
+						pMouseEvent = new QMouseEvent(
+							etype, wpos, pos,
+							Qt::LeftButton,
+							Qt::LeftButton,
+							Qt::NoModifier);
+						QApplication::postEvent(pWidget, pMouseEvent);
+						++iTouched;
+					}
+				}
+			}
+		}
+		// Release all remaining touch-point-ids...
+		etype = QEvent::MouseButtonRelease;
+	#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+		foreach (QEventPoint point, points) {
+			const QPoint& pos = point.position().toPoint();
+	#else
+		foreach (QTouchEvent::TouchPoint point, points) {
+			const QPoint& pos = point.pos().toPoint();
+	#endif
+			const int id = point.id();
+			QWidget *pTouchedWidget = m_touched.value(id, nullptr);
+			if (pTouchedWidget) {
+				const QPoint& wpos
+					= pTouchedWidget->mapFrom(pCentralWidget, pos);
+				pMouseEvent = new QMouseEvent(
+					etype, wpos, pos,
+					Qt::LeftButton,
+					Qt::LeftButton,
+					Qt::NoModifier);
+				QApplication::postEvent(pTouchedWidget, pMouseEvent);
+				++iTouched;
+			}
+		}
+		// Fall thru...
+	case QEvent::TouchCancel:
+		m_touched.clear();
+		m_bSwipe = false;
+		// Fall thru...
+	default:
+		break;
+	}
+
+	// Done with multi-touch stuff.
+	return (iTouched > 0);
+}
+
+
 // end of qmidictlMainForm.cpp
