@@ -1,7 +1,7 @@
 // qmidictlMainForm.h
 //
 /****************************************************************************
-   Copyright (C) 2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2010-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,11 +24,17 @@
 
 #include "ui_qmidictlMainForm.h"
 
+#include <QHash>
 
 // Forward declarations.
 class qmidictlUdpDevice;
 class qmidictlMixerStrip;
 class qmidictlDialStyle;
+#if defined(Q_OS_ANDROID)
+class qmidictlActionBar;
+#endif
+
+class QTouchEvent;
 
 
 //----------------------------------------------------------------------------
@@ -41,7 +47,7 @@ class qmidictlMainForm : public QMainWindow
 public:
 
 	// Constructor.
-	qmidictlMainForm(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
+	qmidictlMainForm(QWidget *pParent = nullptr);
 	// Destructor.
 	~qmidictlMainForm();
 
@@ -103,7 +109,7 @@ protected:
 
 	// MMC Command dispatcher.
 	void sendMmcCommand (MmcCommand cmd,
-		unsigned char *data = NULL, unsigned short len = 0);
+		unsigned char *data = nullptr, unsigned short len = 0);
 
 	// MMC sub-command codes (as for MASKED_WRITE).
 	enum MmcSubCommand {
@@ -150,6 +156,10 @@ protected:
 	// Common form stabilizer.
 	void stabilizeForm();
 
+	// Provided for multi-touch support...
+	bool eventFilter(QObject *pObject, QEvent *pEvent);
+	bool touchEvent(QTouchEvent *pTouchEvent);
+
 protected slots:
 
 	// Mixer strip slots.
@@ -173,7 +183,7 @@ protected slots:
 	void forwardSlot(bool bOn);
 
 	// Network listener/receiver slot.
-	void receiveSlot(const QByteArray& data);
+	void receiveSlot(QByteArray data);
 
 	// Timer slot funtion.
 	void timerSlot();
@@ -223,6 +233,17 @@ private:
 	// Special style for the jog wheel dial.
 	qmidictlDialStyle *m_pDialStyle;
 
+#if defined(Q_OS_ANDROID)
+	// Special action-bar for the android stuff.
+	qmidictlActionBar *m_pActionBar;
+#endif
+
+	// Current touch-point-id to widgets map.
+	QHash<int, QWidget *> m_touched;
+
+	// Custom swipe gesture trackers.
+	bool m_bSwipe;
+
 	// Kind-of singleton reference.
 	static qmidictlMainForm *g_pMainForm;
 };
