@@ -1,7 +1,7 @@
 // qmidictlOptionsForm.h
 //
 /****************************************************************************
-   Copyright (C) 2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2010-2022, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,6 +24,12 @@
 
 #include "ui_qmidictlOptionsForm.h"
 
+// Forward declarations.
+#if defined(Q_OS_ANDROID)
+class qmidictlActionBar;
+class QAction;
+#endif
+
 
 //----------------------------------------------------------------------------
 // qmidictlOptionsForm -- UI wrapper form.
@@ -35,14 +41,20 @@ class qmidictlOptionsForm : public QDialog
 public:
 
 	// Constructor.
-	qmidictlOptionsForm(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
+	qmidictlOptionsForm(QWidget *pParent = nullptr);
+
+	// Destructor.
+	~qmidictlOptionsForm();
 
 protected slots:
 
 	void change();
 
+	void buttonClick(QAbstractButton *);
+
 	void accept();
 	void reject();
+	void reset();
 
 private:
 
@@ -54,6 +66,16 @@ private:
 
 	// Default (translatable) interface name.
 	QString m_sDefInterface;
+
+#if defined(Q_OS_ANDROID)
+	// Special action-bar for the android stuff.
+	qmidictlActionBar *m_pActionBar;
+
+	QAction *m_pBackAction;
+	QAction *m_pResetAction;
+	QAction *m_pAcceptAction;
+	QAction *m_pCancelAction;
+#endif
 };
 
 
